#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
import sys
import re
import pwd
import subprocess
from utils import playbook

if sys.version_info < (3,):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True

#add lib path    
playbook_file = os.path.realpath(sys.argv[0])
playbook_path = os.path.split(playbook_file)[0]   

lib_path=os.path.join(playbook_path,'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0,lib_path)
    lib_path=os.path.join(lib_path,'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0,lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path

_find_unsafe = re.compile(r'[^\w@%+=:,./-]').search

def add_escape(value):
    reserved_chars = r'''$'"'''
    replace = ['\\' + l for l in reserved_chars]
    trans = str.maketrans(dict(zip(reserved_chars, replace)))
    return value.translate(trans)


def quote(s):
    """Return a shell-escaped version of the string *s*."""
    if not s:
        return "''"
    if _find_unsafe(s) is None:
        return s

    # use single quotes, and put single quotes into double quotes
    # the string $'b is then quoted as '$'"'"'b'
    return "'" + add_escape(s) + "'"

class MyPlaybook(playbook.Playbook):

    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('PID','')

    def get_user_name(self, uid):
        users = pwd.getpwall()
        for user in users:
            if user.pw_uid == uid:
                return user.pw_name
        return '%s' % uid

    def run_shell_cmd(self, cmd):
        output = ''
        p = subprocess.Popen(cmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)

        output = p.stdout.read()
        if PY3:
            output = output.decode('utf-8')

        return output
    
    def set_extra_vars_post(self):
        var_redis_pid = self._runcontext.get_extra_var('PID')
        var_redis_pid = self._runcontext.replace_inject_characters(str(var_redis_pid))

        if var_redis_pid == '':
            raise Exception('Must specify extra variable PID.')

        #解析默认配置
        #获取目标文件的属主
        redisservercmd = '/proc/%s/exe' % var_redis_pid
        
        redis_server_running_cmd_uid = os.lstat(redisservercmd).st_uid
        redis_running_user = self.get_user_name(redis_server_running_cmd_uid)
        self._runcontext.set_extra_var('REDIS_REDIS_SERVER_RUNNING_USER', redis_running_user)
        
        redisservercmd = os.readlink(redisservercmd)        
        if os.path.exists(redisservercmd) == False:
            raise Exception('File %s is not exists.' % redisservercmd)
        self._runcontext.log_debug('redis-server = %s' % redisservercmd)
        self._runcontext.set_extra_var('REDIS_REDIS_SERVER', redisservercmd)
        redis_bin = os.path.split(redisservercmd)[0]
        self._runcontext.set_extra_var('REDIS_BIN', redis_bin)

        # check if $REDIS_BIN/redis-cli
        redis_cli = os.path.join(redis_bin, 'redis-cli')
        if not os.path.exists(redis_cli):
            redis_cli = ''
        redis_cli_cmd = redis_cli
         
        if _find_unsafe(redis_cli_cmd) is not None:
            raise Exception('cmd (%s) is unsafe!' % redis_cli_cmd)

        # get redis-cli owner
        redis_cli_owner = pwd.getpwuid(os.stat(redis_cli_cmd).st_uid).pw_name
        
        if redis_cli_owner != 'root':
            cmd = 'su - %s -s /bin/bash -c "%s --help 2>&1 | grep cipherdir"' % (redis_cli_owner, quote(redis_cli))
        else:
            cmd = "{0} --help 2>&1 | grep cipherdir".format(quote(redis_cli))
        output = self.run_shell_cmd(cmd)
        self._runcontext.sleep()
        self._runcontext.log_debug(output)
        if output.strip().startswith('-cipherdir'):
            self._runcontext.set_extra_var('REDIS_ISPAAS_VERSION', True)
        else:
            self._runcontext.set_extra_var('REDIS_ISPAAS_VERSION', False)

        st = os.stat(redisservercmd)
        redisservercmduid = st.st_uid
        redisowner = self.get_user_name(redisservercmduid)
        self._runcontext.set_extra_var('REDIS_REDIS_SERVER_OWNER', redisowner)

        #ps -wp 84465 -o pid,ppid,user,cmd
        #get parent process information
        cmd = 'ps -wp %s -o pid,ppid,user,cmd' % (var_redis_pid)
        output = self.run_shell_cmd(cmd)
        self._runcontext.sleep()
        self._runcontext.log_debug(output)

        #100812  38983 root     ./redis-server 10.175.45.144:7777
        pattern = re.compile(r'%s\s+(\d+)\s+(\S+)\s+(.*)' % var_redis_pid)
        matchs = pattern.findall(output)
        if len(matchs) != 1:
            raise Exception('Could not find process with pid %s' % var_redis_pid)

        var_puser = matchs[0][1]
        var_pcmd = matchs[0][2]
        
        self._runcontext.log_debug('PID : %s' % var_redis_pid)
        self._runcontext.log_debug('PUSER : %s' % var_puser)
        self._runcontext.log_debug('PCMD : %s' % var_pcmd)
        
        self._runcontext.set_extra_var('REDIS_COMMAND', var_pcmd)
        
        # get redis host , port
        if var_pcmd == '':
            raise Exception('Cmd %s is not exists.' % var_pcmd)
        host_port = [x.strip() for x in var_pcmd.split()[1].split(':')]
        
        
        host_port = [self._runcontext.replace_inject_characters(x.strip()) for x in host_port]
        
        self._runcontext.set_extra_var('REDIS_HOST', host_port[0])
        self._runcontext.set_extra_var('REDIS_PORT', host_port[1])
        
        tmp_host = host_port[0]
        if host_port[0] == '*':
            tmp_host = '127.0.0.1'
        redis_cli_cmd = quote(redis_cli_cmd)
        if redis_cli_owner != 'root':
            redis_cli_cmd = 'su - %s -s /bin/bash -c "%s -h %s -p %s"' % (redis_cli_owner, redis_cli_cmd, tmp_host, host_port[1])
        else:
            redis_cli_cmd = '%s -h %s -p %s' % (redis_cli_cmd, tmp_host, host_port[1])
        self._runcontext.set_extra_var('REDIS_CLI_CMD', redis_cli_cmd)
        

    def get_tasks(self, dir):
        ret = []
        ret = []
        ret.append([dir, 'r_2_1_2.py'])
        ret.append([dir, 'r_2_1_3.py'])
        ret.append([dir, 'r_2_1_8.py'])
        ret.append([dir, 'r_2_1_9.py'])
        ret.append([dir, 'r_2_3_1.py'])
        ret.append([dir, 'r_2_3_3.py'])
        ret.append([dir, 'r_3_2_2.py'])
        return ret

if __name__ == '__main__':
    
    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
    

