# -*- coding: UTF-8 -*-
import os
import grp
import pwd
import stat


def get_tags():
    return ['3', '3.2', '3.2.2']
    
    
def run_task(runcontext):
    rc,out = run_audit(runcontext)
    
    return rc,out

    
def run_audit(runcontext):
    curtemplate = runcontext.get_template()
    redis_bin = runcontext.get_extra_var('REDIS_BIN')
    redis_bin = runcontext.replace_inject_characters(redis_bin)
    runcontext.log_debug("redis_bin: %s" % redis_bin)
    check_list = ['redis-benchmark', 'redis-check-aof', 'redis-check-rdb', 'redis-cli', 'redis-server']
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""
    err_msg = []
    system_msg = []
    file_mode = ''
    ret_code = -1
    
    for check in check_list:
        file_mode = ''
        check_final = os.path.join(redis_bin, check)
        if os.path.exists(check_final):
            st = os.stat(check_final)
            
            check_uid = st.st_uid
            check_gid = st.st_gid
            check_user = pwd.getpwuid(check_uid).pw_name
            check_group = grp.getgrgid(check_gid).gr_name
            
            st_mode = st.st_mode
            group_r = bool(st_mode & stat.S_IRGRP)
            group_w = bool(st_mode & stat.S_IWGRP)
            group_x = bool(st_mode & stat.S_IXGRP)
            other_r = bool(st_mode & stat.S_IROTH)
            other_w = bool(st_mode & stat.S_IWOTH)
            other_x = bool(st_mode & stat.S_IXOTH)
            
            file_mode = "%04o" % stat.S_IMODE(st_mode)
            tmp_msg = ''.join([check_user, ':', check_group, '\t', file_mode, '\t', check_final])
            
            system_msg.append(tmp_msg)
            if check_user == 'root' or check_group == 'root' \
                or group_r == True or group_w == True or group_x == True \
                or other_r == True or other_w == True or other_x == True:
                err_msg.append(tmp_msg) 

    if len(err_msg) == 0:
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["policyresult"] = "pass"
        ret_code = 0
        tmp_vars["systemvalue1"] = '\n'.join(system_msg)
    else:
        tmp_vars["systemvalue1"] = '\n'.join(err_msg)

    rc, out = curtemplate.render(runcontext.get_template_file('r_3_2_2.j2'), **tmp_vars)

    if rc == False:
        return (-2,"failed to render template.%s" % out)

    return ret_code,out
